@@ -30,6 +30,7 @@
 #include "file-utils.h"
 #include "fr-archive.h"
 #include "fr-command.h"
+#include "fr-command-arj.h"
 #include "fr-command-cfile.h"
 #include "fr-command-lha.h"
 #include "fr-command-rar.h"
@@ -38,6 +39,8 @@
 #include "fr-command-zoo.h"
 #include "fr-marshal.h"
 #include "fr-process.h"
+#include "utf8-fnmatch.h"
+
 
 #define MAX_CHUNK_LEN 16000 /* FIXME : what is the max length of a command 
 			     * line ? */
@@ -240,7 +243,8 @@ create_command_from_mime_type (FRArchive  *archive,
 		archive->command = fr_command_tar_new (archive->process, 
 						       filename, 
 						       FR_COMPRESS_PROGRAM_BZIP2);
-	} else if (is_mime_type (mime_type, "application/zip")) {
+	} else if (is_mime_type (mime_type, "application/zip") ||
+		   is_mime_type (mime_type, "application/x-zip")) {
 		archive->command = fr_command_zip_new (archive->process, 
 						       filename);
 	} else if (is_mime_type (mime_type, "application/x-zoo")) {
@@ -249,6 +253,9 @@ create_command_from_mime_type (FRArchive  *archive,
 	} else if (is_mime_type (mime_type, "application/x-rar")) {
 		archive->command = fr_command_rar_new (archive->process, 
 						       filename);
+	} else if (is_mime_type (mime_type, "application/x-arj")) {
+		archive->command = fr_command_arj_new (archive->process, 
+						       filename);
 	} else 
 		return FALSE;
 
@@ -378,6 +385,9 @@ create_command_from_filename (FRArchive  *archive,
 	} else if (file_extension_is (filename, ".rar")) {
 		archive->command = fr_command_rar_new (archive->process, 
 						       filename);
+	} else if (file_extension_is (filename, ".arj")) {
+		archive->command = fr_command_arj_new (archive->process, 
+						       filename);
 	} else if (loading) {
 		if (file_extension_is (filename, ".gz")
 		    || file_extension_is (filename, ".z")
@@ -847,14 +857,18 @@ file_list_remove_from_pattern (GList      **list,
 	
 	for (scan = *list; scan;) {
 		char *path = scan->data;
+		char *utf8_name;
 
-		if (match_patterns (patterns, file_name_from_path (path))) {
+		utf8_name = g_locale_to_utf8 (file_name_from_path (path), 
+					      -1, NULL, NULL, NULL);
+		if (match_patterns (patterns, utf8_name, FNM_CASEFOLD)) {
 			*list = g_list_remove_link (*list, scan);
 			g_free (scan->data);
 			g_list_free (scan);
 			scan = *list;
 		} else
 			scan = scan->next;
+		g_free (utf8_name);
 	}
 	
 	g_strfreev (patterns);
@@ -1417,6 +1431,7 @@ G_CONST_RETURN char *
 fr_archive_utils__get_file_name_ext (const char *filename)
 {
 	static char * ext[] = {
+		".arj",
 		".bz", 
 		".bz2", 
 		".ear",
