@@ -26,7 +26,7 @@
 #include <string.h>
 #include <time.h>
 
-#include <gtk/gtk.h>
+#include <glib.h>
 #include <libgnomevfs/gnome-vfs-types.h>
 #include <libgnomevfs/gnome-vfs-mime.h>
 
@@ -35,13 +35,9 @@
 #include "fr-command.h"
 #include "fr-command-cfile.h"
 
-
 static void fr_command_cfile_class_init  (FRCommandCFileClass *class);
-
-static void fr_command_cfile_init        (FRCommand *afile);
-
-static void fr_command_cfile_destroy     (GtkObject *object);
-
+static void fr_command_cfile_init        (FRCommand           *afile);
+static void fr_command_cfile_finalize    (GObject             *object);
 
 /* Parent Class */
 
@@ -49,9 +45,38 @@ static FRCommandClass *parent_class = NULL;
 
 
 static void
-process_line (char *line, gpointer data)
+list__process_line (char     *line, 
+		    gpointer  data)
 {
-	/* never called */
+	FRCommand  *comm = FR_COMMAND (data);
+	FileData   *fdata;
+	char      **fields;
+	char       *filename;
+
+	fdata = file_data_new ();
+
+	fields = split_line (line, 2);
+	fdata->size = atol (fields[1]);
+	g_strfreev (fields);
+
+	filename = remove_extension_from_path (comm->filename);
+	fdata->full_path = g_strconcat ("/", 
+					file_name_from_path (filename),
+					NULL);
+	g_free (filename);
+
+	fdata->original_path = fdata->full_path + 1;
+	fdata->link = NULL;
+	fdata->modified = get_file_mtime (comm->filename);
+	
+	fdata->name = g_strdup (file_name_from_path (fdata->full_path));
+	fdata->path = remove_level_from_path (fdata->full_path);
+	fdata->type = gnome_vfs_mime_type_from_name_or_default (fdata->name, GNOME_VFS_MIME_TYPE_UNKNOWN);
+	
+	if (*fdata->name == 0)
+		file_data_free (fdata);
+	else 
+		comm->file_list = g_list_prepend (comm->file_list, fdata);
 }
 
 
@@ -59,45 +84,69 @@ static void
 fr_command_cfile_list (FRCommand *comm)
 {
 	FRCommandCFile *comm_cfile = FR_COMMAND_CFILE (comm);
-	FileData *fdata;
-	gchar *filename;
 
-	fdata = file_data_new ();
+	if (comm_cfile->compress_prog == FR_COMPRESS_PROGRAM_GZIP) {
+		/* gzip let us known the uncompressed size */
 
-	filename = remove_extension_from_path (comm->filename);
-	fdata->full_path = g_strconcat ("/", 
-					file_name_from_path (filename), 
-					NULL);
-	g_free (filename);
+		fr_process_set_out_line_func (FR_COMMAND (comm)->process, 
+					      list__process_line,
+					      comm);
+		
+		fr_process_clear (comm->process);
+		fr_process_begin_command (comm->process, "gzip");
+		fr_process_add_arg (comm->process, "-l");
+		fr_process_add_arg (comm->process, "-q");
+		fr_process_add_arg (comm->process, comm->e_filename);
+		fr_process_end_command (comm->process);
+		fr_process_start (comm->process);
 
-	fdata->original_path = fdata->full_path + 1;
-	fdata->link = NULL;
-	fdata->size = get_file_size (comm->filename);
-	fdata->modified = get_file_mtime (comm->filename);
+	} else {
+		/* ... other compressors do not support this feature so 
+		 * simply use the archive size, suboptimal but there is no 
+		 * alternative. */
 
-	fdata->name = g_strdup (file_name_from_path (fdata->full_path));
-	fdata->path = remove_level_from_path (fdata->full_path);
-	fdata->type = gnome_vfs_mime_type_from_name_or_default (fdata->name, GNOME_VFS_MIME_TYPE_UNKNOWN);
+		FileData *fdata;
+		char     *filename;
 
-	if (*fdata->name == 0)
-		file_data_free (fdata);
-	else 
-		comm->file_list = g_list_prepend (comm->file_list, fdata);
-	
-	comm_cfile->error.type = FR_PROC_ERROR_NONE;
-	comm_cfile->error.status = 0;
-	gtk_signal_emit_by_name (GTK_OBJECT (comm), 
-				 "done",
-				 comm->action, 
-				 &comm_cfile->error);
+		fdata = file_data_new ();
+
+		filename = remove_extension_from_path (comm->filename);
+		fdata->full_path = g_strconcat ("/", 
+						file_name_from_path (filename),
+						NULL);
+		g_free (filename);
+
+		fdata->original_path = fdata->full_path + 1;
+		fdata->link = NULL;
+		fdata->size = get_file_size (comm->filename);
+		fdata->modified = get_file_mtime (comm->filename);
+		
+		fdata->name = g_strdup (file_name_from_path (fdata->full_path));
+		fdata->path = remove_level_from_path (fdata->full_path);
+		fdata->type = gnome_vfs_mime_type_from_name_or_default (fdata->name, GNOME_VFS_MIME_TYPE_UNKNOWN);
+		
+		if (*fdata->name == 0)
+			file_data_free (fdata);
+		else 
+			comm->file_list = g_list_prepend (comm->file_list, fdata);
+		
+		comm_cfile->error.type = FR_PROC_ERROR_NONE;
+		comm_cfile->error.status = 0;
+		g_signal_emit_by_name (G_OBJECT (comm), 
+				       "done",
+				       comm->action, 
+				       &comm_cfile->error);
+	}
 }
 
 
 static void
-fr_command_cfile_add (FRCommand *comm,
-		      GList *file_list,
-		      gchar *base_dir,
-		      gboolean update)
+fr_command_cfile_add (FRCommand     *comm,
+		      GList         *file_list,
+		      const char    *base_dir,
+		      gboolean       update,
+		      const char    *password,
+		      FRCompression  compression)
 {
 	/* never called */
 }
@@ -105,7 +154,7 @@ fr_command_cfile_add (FRCommand *comm,
 
 static void
 fr_command_cfile_delete (FRCommand *comm,
-			 GList *file_list)
+			 GList     *file_list)
 {
 	/* never called */
 }
@@ -124,19 +173,20 @@ get_temp_dir_name ()
 
 
 static void
-fr_command_cfile_extract (FRCommand *comm,
-			  GList *file_list,
-			  char *dest_dir,
-			  gboolean overwrite,
-			  gboolean skip_older,
-			  gboolean junk_paths)
+fr_command_cfile_extract (FRCommand  *comm,
+			  GList      *file_list,
+			  const char *dest_dir,
+			  gboolean    overwrite,
+			  gboolean    skip_older,
+			  gboolean    junk_paths,
+			  const char *password)
 {
 	FRCommandCFile *comm_cfile = FR_COMMAND_CFILE (comm);
-	char *temp_dir;
-	char *e_temp_dir;
-	char *e_dest_dir;
-	char *temp_file;
-	char *uncompr_file;
+	char           *temp_dir;
+	char           *e_temp_dir;
+	char           *e_dest_dir;
+	char           *temp_file;
+	char           *uncompr_file;
 
 	temp_dir = get_temp_dir_name ();
 	e_temp_dir = shell_escape (temp_dir);
@@ -199,7 +249,10 @@ fr_command_cfile_extract (FRCommand *comm,
 
 	case FR_COMPRESS_PROGRAM_LZOP:
 		fr_process_begin_command (comm->process, "lzop");
-		fr_process_add_arg (comm->process, "-dfU");
+		fr_process_set_working_dir (comm->process, temp_dir);
+		fr_process_add_arg (comm->process, "-d");
+		fr_process_add_arg (comm->process, "-fU");
+		fr_process_add_arg (comm->process, "--no-stdin");
 		fr_process_add_arg (comm->process, temp_file);
 		fr_process_end_command (comm->process);
 		break;
@@ -235,14 +288,13 @@ fr_command_cfile_extract (FRCommand *comm,
 static void 
 fr_command_cfile_class_init (FRCommandCFileClass *class)
 {
+        GObjectClass   *gobject_class = G_OBJECT_CLASS (class);
         FRCommandClass *afc;
-        GtkObjectClass *object_class;
 
-        parent_class = gtk_type_class (FR_COMMAND_TYPE);
-        object_class = (GtkObjectClass*) class;
+        parent_class = g_type_class_peek_parent (class);
 	afc = (FRCommandClass*) class;
 
-        object_class->destroy = fr_command_cfile_destroy;
+        gobject_class->finalize = fr_command_cfile_finalize;
 
         afc->list         = fr_command_cfile_list;
 	afc->add          = fr_command_cfile_add;
@@ -259,61 +311,64 @@ fr_command_cfile_init (FRCommand *comm)
 	comm->propExtractCanAvoidOverwrite = FALSE;
 	comm->propExtractCanSkipOlder      = FALSE;
 	comm->propExtractCanJunkPaths      = FALSE;
+	comm->propPassword                 = FALSE;
+	comm->propTest                     = FALSE;
 }
 
 
 static void 
-fr_command_cfile_destroy (GtkObject *object)
+fr_command_cfile_finalize (GObject *object)
 {
 	FRCommandCFile *comm_tar;
 
         g_return_if_fail (object != NULL);
-        g_return_if_fail (IS_FR_COMMAND_CFILE (object));
+        g_return_if_fail (FR_IS_COMMAND_CFILE (object));
 
 	comm_tar = FR_COMMAND_CFILE (object);
 
 	 /* Chain up */
-        if (GTK_OBJECT_CLASS (parent_class)->destroy)
-                (* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
+        if (G_OBJECT_CLASS (parent_class)->finalize)
+                G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 
-GtkType
+GType
 fr_command_cfile_get_type ()
 {
-        static guint fr_command_cfile_type = 0;
+        static guint type = 0;
 
-        if (!fr_command_cfile_type) {
-                GtkTypeInfo fr_command_cfile_info = {
-                        "FRCommandCFile",
-                        sizeof (FRCommandCFile),
-                        sizeof (FRCommandCFileClass),
-                        (GtkClassInitFunc) fr_command_cfile_class_init,
-                        (GtkObjectInitFunc) fr_command_cfile_init,
-                        /* reserved_1 */ NULL,
-                        /* reserved_2 */ NULL,
-                        (GtkClassInitFunc) NULL,
-                };
-                fr_command_cfile_type = gtk_type_unique(fr_command_get_type(),
-							&fr_command_cfile_info);
+        if (! type) {
+                GTypeInfo type_info = {
+			sizeof (FRCommandCFileClass),
+			NULL,
+			NULL,
+			(GClassInitFunc) fr_command_cfile_class_init,
+			NULL,
+			NULL,
+			sizeof (FRCommandCFile),
+			0,
+			(GInstanceInitFunc) fr_command_cfile_init
+		};
+
+		type = g_type_register_static (FR_TYPE_COMMAND,
+					       "FRCommandCFile",
+					       &type_info,
+					       0);
         }
 
-        return fr_command_cfile_type;
+        return type;
 }
 
 
 FRCommand *
-fr_command_cfile_new (FRProcess *process,
-		      char *filename,
-		      FRCompressProgram prog)
+fr_command_cfile_new (FRProcess         *process,
+		      const char        *filename,
+		      FRCompressProgram  prog)
 {
 	FRCommand *comm;
 
-	comm = FR_COMMAND (gtk_type_new (fr_command_cfile_get_type ()));
+	comm = FR_COMMAND (g_object_new (FR_TYPE_COMMAND_CFILE, NULL));
 	fr_command_construct (comm, process, filename);
-	fr_process_set_proc_line_func (FR_COMMAND (comm)->process, 
-				       process_line,
-				       comm);
 	FR_COMMAND_CFILE (comm)->compress_prog = prog;
 
 	return comm;
